/* Copyright (c) Nordic Semiconductor ASA
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 *   1. Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * 
 *   2. Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 * 
 *   3. Neither the name of Nordic Semiconductor ASA nor the names of other
 *   contributors to this software may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 *   4. This software must only be used in a processor manufactured by Nordic
 *   Semiconductor ASA, or in a processor manufactured by a third party that
 *   is used in combination with a processor manufactured by Nordic Semiconductor.
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <stdint.h>
#include <string.h>
#include "lucka_service.h"
#include "ble_srv_common.h"
#include "app_error.h"
#include "nrf_log.h"
#include "timer_laser.h"
#include "user_flash_storage.h"
//

#define CMD_SAVE 	1
#define CMD_RESET 2

uint8_t char_State=0;
uint8_t char_Time=0;
uint8_t char_Voltage=0;
extern bool _f_switcher_function;
extern void Set_Reset_Values(void);

uint8_t lus_char_value[5];
ble_gatts_value_t lus_char;
uint32_t err_tmp;

static void on_write(ble_lus_t * p_lus, ble_evt_t * p_ble_evt)
{
	ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;
	if ((p_evt_write->handle == p_lus->char_handles_freq.value_handle)&&(p_evt_write->len == 1))
	{
    NRF_LOG_DEBUG("Freq value: %ukHz\r\n", p_evt_write->data[0]);
		Laser_Freq = p_evt_write->data[0];
		lus_char.p_value = lus_char_value;
		err_tmp=sd_ble_gatts_value_get(p_lus->conn_handle, p_lus->char_handles_freq.value_handle, &lus_char);	
    NRF_LOG_DEBUG("Char Value: %u Err: %u\r\n", lus_char.p_value[0], err_tmp);
	}
	if ((p_evt_write->handle == p_lus->char_handles_time.value_handle)&&(p_evt_write->len == 1))
	{
    NRF_LOG_DEBUG("Terapy time: %us\r\n", p_evt_write->data[0]);
    char_Time	= p_evt_write->data[0];	
	}
	if ((p_evt_write->handle == p_lus->char_handles_mode.value_handle)&&(p_evt_write->len == 1))
	{
    NRF_LOG_DEBUG("Mode: %u\r\n", p_evt_write->data[0]);
    _f_switcher_function	= p_evt_write->data[0];				
	}
	if ((p_evt_write->handle == p_lus->char_handles_cmd.value_handle)&&(p_evt_write->len == 1))
	{
    NRF_LOG_DEBUG("Command: %u\r\n", p_evt_write->data[0]);
    switch(p_evt_write->data[0]){
			case CMD_SAVE: 
				fds_param_write();
				break;
			case CMD_RESET:
				fds_param_delete();	
			  Set_Reset_Values();
			  lus_char.len=1;
			  lus_char.offset=0;
			  lus_char.p_value = lus_char_value;
			  lus_char_value[0]=RESET_FREQUENCY;
			  sd_ble_gatts_value_set(p_lus->conn_handle, p_lus->char_handles_freq.value_handle, &lus_char);
			  lus_char_value[0]=RESET_TIME;
			  sd_ble_gatts_value_set(p_lus->conn_handle, p_lus->char_handles_time.value_handle, &lus_char);	
			  lus_char_value[0]=RESET_MODE;
			  sd_ble_gatts_value_set(p_lus->conn_handle, p_lus->char_handles_mode.value_handle, &lus_char);				
				break;
		}			
	} 	
}

// ALREADY_DONE_FOR_YOU: Declaration of a function that will take care of some housekeeping of ble connections related to our service and characteristic
void ble_lucka_service_on_ble_evt(ble_lus_t * p_lus, ble_evt_t * p_ble_evt)
{
    // OUR_JOB: Step 3.D Implement switch case handling BLE events related to our service. 
	if(p_ble_evt->header.evt_id!=1){
		NRF_LOG_DEBUG("Lucka Event: 0x%X\r\n",p_ble_evt->header.evt_id);
	}
	switch (p_ble_evt->header.evt_id){
    case BLE_GAP_EVT_CONNECTED:
        p_lus->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
        break;
    case BLE_GAP_EVT_DISCONNECTED:
        p_lus->conn_handle = BLE_CONN_HANDLE_INVALID;
        break;
		case BLE_GATTS_EVT_WRITE:
				on_write(p_lus, p_ble_evt);		
        break;
    default:
        // No implementation needed.
        break;
	}
}

/**@brief Function for adding our new characterstic to "Our service" that we initiated in the previous tutorial. 
 *
 * @param[in]   p_our_service        Our Service structure.
 *
 */
static uint32_t lucka_char_add(ble_lus_t * p_lucka_service)
{   
    uint32_t     		err_code;
		ble_uuid_t     	char_uuid;
		ble_uuid128_t  	base_uuid = BLE_UUID_LUCKA_BASE;
	  ble_gatts_attr_md_t cccd_md;
	  ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t attr_md;	
		ble_gatts_attr_t    attr_char_value;

//////////////CHARASTERICTIC_1 - STATE///////////////////////////////////////	
	  // Add a custom characteristic UUID
		char_uuid.uuid  = BLE_UUID_LUCKA_CHAR_STATE;
		err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
		APP_ERROR_CHECK(err_code);
	
    // Add read/write properties to our characteristic   
    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.read 		= 1;
		//char_md.char_props.write 		= 1;
		char_md.p_cccd_md           = &cccd_md;
		char_md.char_props.notify   = 1;
    
    // Configuring Client Characteristic Configuration Descriptor metadata and add to char_md structure
    memset(&cccd_md, 0, sizeof(cccd_md));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
		cccd_md.vloc = BLE_GATTS_VLOC_STACK; 		
    
    // Configure the attribute metadata
    memset(&attr_md, 0, sizeof(attr_md));  
    attr_md.vloc = BLE_GATTS_VLOC_STACK;  
    // Set read/write security levels to our characteristic
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
		//BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // Configure the characteristic value attribute    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md; 
    // Set characteristic length in number of bytes
    attr_char_value.max_len     = 1;
		attr_char_value.init_len    = 1;
		attr_char_value.p_value     = &char_State;
		
    // Add our new characteristic to the service
    err_code = sd_ble_gatts_characteristic_add(p_lucka_service->service_handle,
																							 &char_md,
																							 &attr_char_value,
																							 &p_lucka_service->char_handles_state);
		APP_ERROR_CHECK(err_code);
////////////////////////////////////////////////////////////////////////////////////////////
		
//////////////CHARASTERICTIC_2 - FREQUENCY ///////////////////////////////////////	
	  // Add a custom characteristic UUID
		char_uuid.uuid  = BLE_UUID_LUCKA_CHAR_FREQ;
		err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
		APP_ERROR_CHECK(err_code);
	
    // Add read/write properties to our characteristic   
    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.read 		= 1;
		char_md.char_props.write 		= 1;		
    
    // Configure the attribute metadata
    memset(&attr_md, 0, sizeof(attr_md));  
    attr_md.vloc = BLE_GATTS_VLOC_STACK;  
    // Set read/write security levels to our characteristic
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // Configure the characteristic value attribute    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md; 
    // Set characteristic length in number of bytes
    attr_char_value.max_len     = 1;
		attr_char_value.init_len    = 1;
		attr_char_value.p_value     = &Laser_Freq;//value_char_freq;//
		
    // Add our new characteristic to the service
    err_code = sd_ble_gatts_characteristic_add(p_lucka_service->service_handle,
																							 &char_md,
																							 &attr_char_value,
																							 &p_lucka_service->char_handles_freq);
		APP_ERROR_CHECK(err_code);
////////////////////////////////////////////////////////////////////////////////////////////		

//////////////CHARASTERICTIC_3 - TIMER ///////////////////////////////////////	
	  // Add a custom characteristic UUID
		char_uuid.uuid  = BLE_UUID_LUCKA_CHAR_TIMER;
		err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
		APP_ERROR_CHECK(err_code);
	
    // Add read/write properties to our characteristic   
    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.read 		= 1;
		char_md.char_props.write 		= 1;		
    
    // Configure the attribute metadata
    memset(&attr_md, 0, sizeof(attr_md));  
    attr_md.vloc = BLE_GATTS_VLOC_STACK;  
    // Set read/write security levels to our characteristic
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // Configure the characteristic value attribute    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md; 
    // Set characteristic length in number of bytes
    attr_char_value.max_len     = 1;
		attr_char_value.init_len    = 1;
		attr_char_value.p_value     = &char_Time;
		
    // Add our new characteristic to the service
    err_code = sd_ble_gatts_characteristic_add(p_lucka_service->service_handle,
																							 &char_md,
																							 &attr_char_value,
																							 &p_lucka_service->char_handles_time);
		APP_ERROR_CHECK(err_code);
////////////////////////////////////////////////////////////////////////////////////////////		

//////////////CHARASTERICTIC_4 - VOLTAGE///////////////////////////////////////	
	  // Add a custom characteristic UUID
		char_uuid.uuid  = BLE_UUID_LUCKA_CHAR_VOLTAGE;
		err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
		APP_ERROR_CHECK(err_code);
	
    // Add read/write properties to our characteristic   
    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.read 		= 1;
		char_md.char_props.write 		= 1;		
    
    // Configure the attribute metadata
    memset(&attr_md, 0, sizeof(attr_md));  
    attr_md.vloc = BLE_GATTS_VLOC_STACK;  
    // Set read/write security levels to our characteristic
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // Configure the characteristic value attribute    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md; 
    // Set characteristic length in number of bytes
    attr_char_value.max_len     = 1;
		attr_char_value.init_len    = 1;
		attr_char_value.p_value     = &char_Voltage;//value_char_freq;//
		
    // Add our new characteristic to the service
    err_code = sd_ble_gatts_characteristic_add(p_lucka_service->service_handle,
																							 &char_md,
																							 &attr_char_value,
																							 &p_lucka_service->char_handles_voltage);
		APP_ERROR_CHECK(err_code);
////////////////////////////////////////////////////////////////////////////////////////////

//////////////CHARASTERICTIC_5 - MODE ///////////////////////////////////////	
	  // Add a custom characteristic UUID
		char_uuid.uuid  = BLE_UUID_LUCKA_CHAR_MODE;
		err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
		APP_ERROR_CHECK(err_code);
	
    // Add read/write properties to our characteristic   
    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.read 		= 1;
		char_md.char_props.write 		= 1;		
    
    // Configure the attribute metadata
    memset(&attr_md, 0, sizeof(attr_md));  
    attr_md.vloc = BLE_GATTS_VLOC_STACK;  
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // Configure the characteristic value attribute    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md; 
    attr_char_value.max_len     = 1;
		attr_char_value.init_len    = 1;
		attr_char_value.p_value     = (uint8_t *)&_f_switcher_function;
		
    // Add our new characteristic to the service
    err_code = sd_ble_gatts_characteristic_add(p_lucka_service->service_handle,
																							 &char_md,
																							 &attr_char_value,
																							 &p_lucka_service->char_handles_mode);
		APP_ERROR_CHECK(err_code);
////////////////////////////////////////////////////////////////////////////////////////////

//////////////CHARASTERICTIC_6 - COMMAND ///////////////////////////////////////	
	  // Add a custom characteristic UUID
		char_uuid.uuid  = BLE_UUID_LUCKA_CHAR_CMD;
		err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
		APP_ERROR_CHECK(err_code);
	
    // Add read/write properties to our characteristic   
    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.write 		= 1;		
    
    // Configure the attribute metadata
    memset(&attr_md, 0, sizeof(attr_md));  
    attr_md.vloc = BLE_GATTS_VLOC_STACK;  
		BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    
    // Configure the characteristic value attribute    
    memset(&attr_char_value, 0, sizeof(attr_char_value));
		attr_char_value.p_uuid      = &char_uuid;
    attr_char_value.p_attr_md   = &attr_md; 
    attr_char_value.max_len     = 1;
		attr_char_value.init_len    = 1;
		
    // Add our new characteristic to the service
    err_code = sd_ble_gatts_characteristic_add(p_lucka_service->service_handle,
																							 &char_md,
																							 &attr_char_value,
																							 &p_lucka_service->char_handles_cmd);
		APP_ERROR_CHECK(err_code);
////////////////////////////////////////////////////////////////////////////////////////////


    return NRF_SUCCESS;
}



/**@brief Function for initiating our new service.
 *
 * @param[in]   p_our_service        Our Service structure.
 *
 */
void lucka_service_init(ble_lus_t * p_lucka_service)
{
  //Declare 16 bit service and 128 bit base UUIDs and add them to BLE stack table     
	uint32_t err_code;
	ble_uuid_t service_uuid;
	ble_uuid128_t base_uuid = BLE_UUID_LUCKA_BASE;
	
	service_uuid.uuid = BLE_UUID_LUCKA_SERVICE ;
	err_code = sd_ble_uuid_vs_add(&base_uuid, &service_uuid.type);
	APP_ERROR_CHECK(err_code);
	
	p_lucka_service->conn_handle = BLE_CONN_HANDLE_INVALID;
  //Add lucka service
	err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY,
                                    &service_uuid,
                                    &p_lucka_service->service_handle);
	APP_ERROR_CHECK(err_code);

  // OUR_JOB: Call the function our_char_add() to add our new characteristic to the service. 
  lucka_char_add(p_lucka_service);
}

// ALREADY_DONE_FOR_YOU: Function to be called when updating characteristic value
void lucka_state_char_update(ble_lus_t *p_lus, uint8_t state)
{
    // OUR_JOB: Step 3.E, Update characteristic value
	if (p_lus->conn_handle != BLE_CONN_HANDLE_INVALID)
	{
		uint16_t  len = 1;
		uint8_t 	LaserState = state;
		ble_gatts_hvx_params_t hvx_params;
		memset(&hvx_params, 0, sizeof(hvx_params));

		hvx_params.handle = p_lus->char_handles_state.value_handle;
		hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
		hvx_params.offset = 0;
		hvx_params.p_len  = &len;
		hvx_params.p_data = &LaserState;  

		sd_ble_gatts_hvx(p_lus->conn_handle, &hvx_params);
	}
	char_State=state;

}

//		ble_gatts_attr_md_t    attr_md;
//    memset(&attr_md, 0, sizeof(attr_md));

//    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
//    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&attr_md.write_perm);

//    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
//    attr_md.rd_auth = 0;
//    attr_md.wr_auth = 0;
//    attr_md.vlen    = 0;
//		
//    uint8_t attr_value[]  = "A";
//    const uint16_t attr_len   = sizeof(attr_value);
//		
//		//ble_uuid_t          ble_uuid;
//		//BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_REPORT_REF_DESCR);//BLE_UUID_DESCRIPTOR_CHAR_USER_DESC);// BLE_UUID_REPORT_REF_DESCR);
//    ble_uuid_t          attr_uuid;
//    attr_uuid.uuid      = BLE_UUID_DESCRIPTOR_CHAR_USER_DESC;
//    err_code = sd_ble_uuid_vs_add(&base_uuid, &attr_uuid.type);
//    APP_ERROR_CHECK(err_code); 
//    ble_gatts_attr_t    attr;
//    memset(&attr, 0, sizeof(attr));
//		
//    attr.init_len       = attr_len;
//    attr.max_len        = attr_len;
//    attr.init_offs      = 0;
//    attr.p_value        = attr_value;
//    attr.p_attr_md      = &attr_md;
//    attr.p_uuid         = &attr_uuid;

//    err_code = sd_ble_gatts_descriptor_add(p_lucka_service->char_handles_freq.value_handle , &attr, &p_lucka_service->descr_handle);
//    APP_ERROR_CHECK(err_code); 
